"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Socket = void 0;
const lodash_1 = require("lodash");
const queues_cache_1 = require("./queues-cache");
const ws_autoreconnect_1 = require("./ws-autoreconnect");
const queue_factory_1 = require("./queue-factory");
const utils_1 = require("./utils");
const { version } = require(`${__dirname}/../package.json`);
const chalk = require("chalk");
const Socket = (name, server, token, connection, opts = {}) => {
    const { team, nodes, integrations } = opts;
    const ws = new ws_autoreconnect_1.WebSocketClient();
    const redisOpts = redisOptsFromConnection(connection);
    ws.open(server, {
        headers: {
            Authorization: "Bearer " + token,
            Taskforce: "connector",
        },
    });
    console.log(`${chalk.yellow("WebSocket:")} ${chalk.blueBright("opening connection to")} ${chalk.gray("Taskforce.sh")} (${chalk.blueBright(server)})`);
    ws.onopen = function open() {
        console.log(chalk.yellow("WebSocket:") +
            chalk.blueBright(" opened connection to ") +
            chalk.gray("Taskforce.sh"));
    };
    ws.onerror = function (err) {
        var msg;
        if (err.message === "Unexpected server response: 401") {
            msg =
                "Authorization failed, please check that you are using the correct token from your account page";
        }
        else {
            msg = err.message;
        }
        console.log(chalk.yellow("WebSocket: ") + chalk.red(msg));
    };
    ws.onmessage = function incoming(input) {
        return __awaiter(this, void 0, void 0, function* () {
            console.log(`${chalk.yellow("WebSocket:")} ${chalk.blueBright("received")} %s`, input);
            try {
                if (input === "authorized") {
                    console.log(chalk.yellow("WebSocket: ") +
                        chalk.green("Succesfully authorized to taskforce.sh service"));
                    const queues = yield (0, queues_cache_1.updateQueuesCache)(redisOpts, {
                        nodes,
                        integrations,
                    });
                    console.log(`${chalk.yellow("WebSocket: ")} ${chalk.green("sending connection: ")} ${chalk.blueBright(name)} ${team ? chalk.green(" for team ") + chalk.blueBright(team) : ""}`);
                    ws.send(JSON.stringify({
                        res: "connection",
                        cmd: "update",
                        queues,
                        connection: name,
                        team,
                        version,
                    }));
                }
                else {
                    const msg = JSON.parse(input);
                    if (!msg.data) {
                        console.error(chalk.red("WebSocket:") +
                            chalk.blueBright(" missing message data "), msg);
                        return;
                    }
                    const { res, queueName, queuePrefix } = msg.data;
                    switch (res) {
                        case "connections":
                            yield respondConnectionCommand(connection, msg);
                            break;
                        case "queues":
                        case "jobs":
                            const cache = (0, queues_cache_1.getCache)();
                            if (!cache) {
                                yield (0, queues_cache_1.updateQueuesCache)(redisOpts, opts);
                            }
                            const { queue, responders } = cache[(0, queues_cache_1.queueKey)({ name: queueName, prefix: queuePrefix || "bull" })];
                            if (!queue) {
                                ws.send(JSON.stringify({
                                    id: msg.id,
                                    err: "Queue not found",
                                }));
                            }
                            else {
                                switch (res) {
                                    case "queues":
                                        yield responders.respondQueueCommand(ws, queue, msg);
                                        break;
                                    case "jobs":
                                        yield responders.respondJobCommand(ws, queue, msg);
                                        break;
                                }
                            }
                            break;
                    }
                }
            }
            catch (err) {
                console.error(err);
            }
        });
    };
    function respondConnectionCommand(connection, msg) {
        return __awaiter(this, void 0, void 0, function* () {
            const data = msg.data;
            switch (data.cmd) {
                case "ping":
                    const pong = yield (0, queue_factory_1.ping)(redisOpts, nodes);
                    respond(msg.id, pong);
                    break;
                case "getConnection":
                    {
                        const queues = yield (0, queues_cache_1.updateQueuesCache)(redisOpts, opts);
                        console.log(`${chalk.yellow("WebSocket: ")} ${chalk.green("sending connection:")} ${chalk.blueBright(name)} ${team ? chalk.green(" for team ") + chalk.blueBright(team) : ""}`);
                        respond(msg.id, {
                            queues,
                            connection: name,
                            team,
                            version,
                        });
                    }
                    break;
                case "getQueues":
                    {
                        const queues = yield (0, queues_cache_1.updateQueuesCache)(redisOpts, opts);
                        for (const queue of queues) {
                            const { name, prefix, type } = queue;
                            console.log(`${chalk.yellow("WebSocket:")} ${chalk.blueBright("Sending queue:")} ${chalk.green(name)} ${chalk.blueBright("type:")} ${chalk.green(type)} ${chalk.blueBright("prefix:")} ${chalk.green(prefix)}`);
                        }
                        respond(msg.id, queues);
                    }
                    break;
                case "getInfo":
                    const info = yield (0, queue_factory_1.getRedisInfo)(redisOpts, nodes);
                    respond(msg.id, info);
                    break;
                case "getQueueType":
                    const queueType = yield (0, queue_factory_1.execRedisCommand)(redisOpts, (client) => (0, utils_1.getQueueType)(data.name, data.prefix, client), nodes);
                    respond(msg.id, { queueType });
                    break;
            }
        });
    }
    function respond(id, data = {}) {
        const response = JSON.stringify({
            id,
            data,
        });
        ws.send(response);
    }
};
exports.Socket = Socket;
function redisOptsFromConnection(connection) {
    let opts = Object.assign({}, (0, lodash_1.pick)(connection, [
        "host",
        "port",
        "username",
        "password",
        "family",
        "sentinelPassword",
        "db",
        "tls",
        "sentinels",
        "name",
    ]));
    if (connection.uri) {
        opts = Object.assign(Object.assign({}, opts), (0, utils_1.redisOptsFromUrl)(connection.uri));
    }
    opts.retryStrategy = function (times) {
        times = times % 8;
        const delay = Math.round(Math.pow(2, times + 8));
        console.log(chalk.yellow("Redis: ") + `Reconnecting in ${delay} ms`);
        return delay;
    };
    return opts;
}
