"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebSocketClient = void 0;
const WebSocket = require("ws");
const chalk = require("chalk");
const ws_errors_enum_1 = require("./ws-errors.enum");
const HEARTBEAT_INTERVAL = 15000;
class WebSocketClient {
    constructor() {
        this.number = 0;
        this.autoReconnectInterval = 5 * 1000;
        this.onmessage = function (data, flags, num) {
            console.log("WebSocket: message", data, flags, num);
        };
        this.onerror = function (e) {
            console.log("WebSocket: error", arguments);
        };
        this.onclose = function (e) {
            console.log("WebSocket: closed", arguments);
        };
    }
    open(url, opts) {
        this.url = url;
        this.opts = opts;
        this.instance = new WebSocket(url, opts);
        this.instance.on("open", () => {
            this.heartbeat();
            this.onopen();
        });
        this.instance.on("message", (data, flags) => {
            this.number++;
            this.onmessage(data, flags, this.number);
        });
        this.instance.on("ping", () => this.heartbeat());
        this.instance.on("close", (codeOrError) => {
            clearTimeout(this.pingTimeout);
            switch (codeOrError) {
                case ws_errors_enum_1.WebsocketError.NormalClosure:
                    console.log(chalk.yellow("WebSocket:") + chalk.blue("normally closed"));
                    break;
                case 4000:
                    console.log(chalk.yellow("WebSocket:") + chalk.red(" Invalid authentication"));
                    break;
                default:
                    this.reconnect(codeOrError);
                    break;
            }
            this.onclose(codeOrError);
        });
        this.instance.on("error", (err) => {
            switch (err["code"]) {
                case "ECONNREFUSED":
                    break;
                default:
                    this.onerror(err);
                    break;
            }
        });
    }
    send(data, option) {
        try {
            this.instance.send(data, option);
        }
        catch (err) {
            this.instance.emit("error", err);
        }
    }
    reconnect(err) {
        var msg = err.message || "";
        console.log(chalk.yellow("WebSocket:") +
            chalk.red(` ${msg} retry in ${this.autoReconnectInterval}ms`));
        this.instance.removeAllListeners();
        setTimeout(() => {
            console.log("WebSocket: reconnecting...");
            this.open(this.url, this.opts);
        }, this.autoReconnectInterval);
    }
    onopen() {
        console.log("WebSocket: open", arguments);
    }
    heartbeat() {
        clearTimeout(this.pingTimeout);
        this.pingTimeout = setTimeout(() => {
            this.instance.terminate();
        }, HEARTBEAT_INTERVAL);
    }
}
exports.WebSocketClient = WebSocketClient;
